searchData={"items":[{"type":"module","doc":"Exceed is a high-level stream-oriented library for generating Excel files,\nuseful when generating spreadsheets from data sets that exceed available\nmemory (or the memory that one wishes to devote to generating Excel files).","title":"Exceed","ref":"Exceed.html"},{"type":"module","doc":"``` elixir\niex> rows = Stream.repeatedly(fn -> [:rand.uniform(), :rand.uniform()] end)\niex> stream = Exceed.Workbook.new(\"creator name\")\n...>   |> Exceed.Workbook.add_worksheet(\n...>     Exceed.Worksheet.new(\"Sheet\", [\"header a\", \"header b\"], Enum.take(rows, 10)))\n...>   |> Exceed.stream!()\n...>\niex> zip = stream |> Enum.to_list() |> IO.iodata_to_binary()\niex> {:ok, package} = XlsxReader.open(zip, [source: :binary])\niex> XlsxReader.sheet_names(package)\n[\"Sheet\"]\n```","title":"Examples - Exceed","ref":"Exceed.html#module-examples"},{"type":"function","doc":"Convert an `Exceed.Workbook` to a stream. See `Exceed.Workbook.new/1`,\n`Exceed.Worksheet.new/4`, and `Exceed.Workbook.add_worksheet/2`.\n\nThe only option at the moment is `buffer` which can be set to `true` (the default)\nor to `false` (which may be more performant in some situations).","title":"Exceed.stream!/2","ref":"Exceed.html#stream!/2"},{"type":"module","doc":"The top-level data structure that collects worksheets and metadata for\ngenerating an Excel file.","title":"Exceed.Workbook","ref":"Exceed.Workbook.html"},{"type":"module","doc":"``` elixir\niex> Exceed.Workbook.new(\"creator name\")\n#Exceed.Workbook \n```\n\n``` elixir\niex> headers = [\"header 1\"]\niex> rows = Stream.repeatedly(fn -> [:rand.uniform(), :rand.uniform()] end)\niex> ws = Exceed.Worksheet.new(\"Sheet Name\", headers, rows)\n...>\niex> Exceed.Workbook.new(\"creator name\")\n...>   |> Exceed.Workbook.add_worksheet(ws)\n#Exceed.Workbook \n```","title":"Examples - Exceed.Workbook","ref":"Exceed.Workbook.html#module-examples"},{"type":"function","doc":"Adds an `Exceed.Worksheet` to the workbook.","title":"Exceed.Workbook.add_worksheet/2","ref":"Exceed.Workbook.html#add_worksheet/2"},{"type":"function","doc":"Initialize a new workbook with a creator name.","title":"Exceed.Workbook.new/1","ref":"Exceed.Workbook.html#new/1"},{"type":"type","doc":"","title":"Exceed.Workbook.t/0","ref":"Exceed.Workbook.html#t:t/0"},{"type":"module","doc":"Worksheets represent the tabular data to be included in an Excel sheet, in\naddition to metadata about the sheet and how it should be rendered.","title":"Exceed.Worksheet","ref":"Exceed.Worksheet.html"},{"type":"module","doc":"``` elixir\niex> headers = [\"header 1\"]\niex> rows = [[\"row 1\"], [\"row 2\"]]\niex> ws = Exceed.Worksheet.new(\"Sheet Name\", headers, rows)\n#Exceed.Worksheet \niex>\niex> Exceed.Workbook.new(\"creator name\")\n...>  |> Exceed.Workbook.add_worksheet(ws)\n#Exceed.Workbook \n```","title":"Examples - Exceed.Worksheet","ref":"Exceed.Worksheet.html#module-examples"},{"type":"module","doc":"Rows are represented by an enumerable where each row will be resolved to a\nlist of cells. In the above example, a list of lists is provided.\nAlternatively, a stream may be provided.\n\n``` elixir\niex> stream = Stream.repeatedly(fn -> [:rand.uniform(), :rand.uniform()] end)\n...>\niex> Exceed.Worksheet.new(\"Sheet Name\", [\"Random 1\", \"Random 2\"], stream)\n#Exceed.Worksheet \n```\n\nValues in each row execute the `Exceed.Worksheet.Cell` protocol to convert\nElixir data structures to XML using appropriate SpreadsheetML tags and\ndetermine the appropriate XML attributes to merge onto the cell.","title":"Sheet content - Exceed.Worksheet","ref":"Exceed.Worksheet.html#module-sheet-content"},{"type":"module","doc":"When initializing a worksheet, default options may be overridded by providing\nan options via the optional fourth argument to `Exceed.Worksheet.new/4`.\n\n``` elixir\niex> headers = [\"header 1\"]\niex> rows = [[\"row 1\"], [\"row 2\"]]\niex> opts = [cols: [padding: 6.325, widths: %{2 => 10.75}]]\niex>\niex> Exceed.Worksheet.new(\"Sheet Name\", headers, rows, opts)\n#Exceed.Worksheet \n```\n\n- Column padding - `cols: [padding: value]` - default: `4.25` - extra space\n  given to each column, in addition to whatever is determined from the\n  headers or the specified widths. Not used when an exact width is specified\n  for a column.\n- Column width - `cols: [widths: %{1 => 15.75}]` - specify the exact width\n  of specific columns as a map of 1-indexed column indexes to floats. When not\n  provided, this is automatically determined from the character length of the\n  relevant header cell, or from the first row when no headers are provided.","title":"Sheet options - Exceed.Worksheet","ref":"Exceed.Worksheet.html#module-sheet-options"},{"type":"module","doc":"1. Note that the actual rendered width of a cell depends on the character\n  width of the applied font, on the computer used to open the spreadsheet (see\n  the [OpenXML.Spreadsheet Column docs](https://learn.microsoft.com/en-us/dotnet/api/documentformat.openxml.spreadsheet.column?view=openxml-3.0.1)\n  for more info).","title":"Column widths - Exceed.Worksheet","ref":"Exceed.Worksheet.html#module-column-widths"},{"type":"type","doc":"","title":"Exceed.Worksheet.columns_option/0","ref":"Exceed.Worksheet.html#t:columns_option/0"},{"type":"type","doc":"","title":"Exceed.Worksheet.headers/0","ref":"Exceed.Worksheet.html#t:headers/0"},{"type":"function","doc":"Initialize a new worksheet to be added to a workbook. See `Exceed.Workbook.add_worksheet/2`.\nFor worksheet options, see the module docs for `m:Exceed.Worksheet#module-sheet-options`.","title":"Exceed.Worksheet.new/4","ref":"Exceed.Worksheet.html#new/4"},{"type":"function","doc":"``` elixir\niex> headers = [\"header 1\"]\niex> rows = [[\"row 1\"], [\"row 2\"]]\niex> opts = [cols: [padding: 6.325]]\niex> %Worksheet{} = Exceed.Worksheet.new(\"Sheet Name\", headers, rows, opts)\n\n```","title":"Examples - Exceed.Worksheet.new/4","ref":"Exceed.Worksheet.html#new/4-examples"},{"type":"type","doc":"","title":"Exceed.Worksheet.spreadsheet_option/0","ref":"Exceed.Worksheet.html#t:spreadsheet_option/0"},{"type":"type","doc":"","title":"Exceed.Worksheet.spreadsheet_options/0","ref":"Exceed.Worksheet.html#t:spreadsheet_options/0"},{"type":"type","doc":"","title":"Exceed.Worksheet.t/0","ref":"Exceed.Worksheet.html#t:t/0"},{"type":"protocol","doc":"A protocol for transforming source data into data structures that can be streamed\nto appropriate SpreadsheetML tags, using the `XmlStream` library.\n\nThis protocol is implemented for floats, integers, strings, and binaries, in addition\nto `Date`, `DateTime`, and `NaiveDateTime`. If the [decimal](https://hex.pm/packages/decimal)\nlibrary is present, this protocoal is automatically implemented for `Decimal`.","title":"Exceed.Worksheet.Cell","ref":"Exceed.Worksheet.Cell.html"},{"type":"protocol","doc":"``` elixir\ndefimpl Exceed.Worksheet.Cell, for: MyStruct do\n  alias XmlStream, as: Xs\n\n  def to_attrs(%MyStruct{value: value}) when is_binary(value),\n    do: %{\"t\" => \"inlineStr\"}\n\n  def to_content(%MyStruct{value: value}) when is_binary(value),\n    do: Xs.element(\"is\", [Xs.element(\"t\", [Xs.content(value)])])\nend\n```","title":"Examples - Exceed.Worksheet.Cell","ref":"Exceed.Worksheet.Cell.html#module-examples"},{"type":"type","doc":"All the types that implement this protocol.","title":"Exceed.Worksheet.Cell.t/0","ref":"Exceed.Worksheet.Cell.html#t:t/0"},{"type":"function","doc":"For a given data type, these attributes will be merged onto the `c` tag wrapping\nthis cell's content. Note that the `r` attribute (designating the cell's identifier\nin `A1` format) will be calculated when streaming a worksheet to XLSX, and should\n_not_ be included in this output.","title":"Exceed.Worksheet.Cell.to_attrs/1","ref":"Exceed.Worksheet.Cell.html#to_attrs/1"},{"type":"function","doc":"For a given data type, convert the value to a list of tags. Functions from\n`XmlStream` including `XmlStream.element/3`, `XmlStream.empty_element/2`, and\n`XmlStream.content/1` may be used to facilitate the generation of tags.","title":"Exceed.Worksheet.Cell.to_content/1","ref":"Exceed.Worksheet.Cell.html#to_content/1"},{"type":"module","doc":"Helpers for converting Elixir data formats to Excel.","title":"Exceed.Util","ref":"Exceed.Util.html"},{"type":"type","doc":"","title":"Exceed.Util.erl_datetime_t/0","ref":"Exceed.Util.html#t:erl_datetime_t/0"},{"type":"function","doc":"Converts a `Date`, `DateTime`, or `NaiveDateTime` to a float representing days\nsince 1900, correcting for the Lotus 123 bug (Excel treats 1900 as a leap\nyear).","title":"Exceed.Util.to_excel_datetime/1","ref":"Exceed.Util.html#to_excel_datetime/1"},{"type":"function","doc":"``` elixir\n```","title":"Examples - Exceed.Util.to_excel_datetime/1","ref":"Exceed.Util.html#to_excel_datetime/1-examples"},{"type":"exception","doc":"","title":"Exceed.Error","ref":"Exceed.Error.html"},{"type":"extras","doc":"# Phoenix Integration\n\nGiven a Phoenix application, Excel files streamed via Exceed may be downloaded from controllers.\nThis requires that the `conn` be configured as a chunked, and then each chunk of the stream be\nreduced into it.","title":"Phoenix Integration","ref":"phoenix.html"},{"type":"extras","doc":"``` elixir\ndefmodule Web.ExcelController do\n  use Web, :controller\n\n  def download(conn, _params) do\n    conn =\n      conn\n      |> put_resp_content_type(\"application/vnd.openxmlformats-officedocument.spreadsheetml.sheet\")\n      |> put_resp_header(\"content-disposition\", \"attachment; filename=file.xlsx\")\n      |> send_chunked(:ok)\n\n    excel_stream()\n    |> Enum.reduce_while(conn, fn excel_chunk, conn ->\n      case chunk(conn, excel_chunk) do\n        {:ok, conn} -> {:cont, conn}\n        {:error, :closed} -> {:halt, conn}\n      end\n    end)\n  end\n\n  # # #\n\n  defp excel_stream do\n    Exceed.Workbook.new(\"Creator Name\")\n    |> Exceed.Workbook.add_worksheet(\n      Exceed.Worksheet.new(\"Sheet Name\", [\"Heading 1\", \"Heading 2\"],\n        [[\"Row 1 Cell 1\", \"Row 1 Cell 2\"], [\"Row 2 Cell 1\", \"Row 2 Cell 2\"]])\n    )\n    |> Exceed.stream!()\n  end\nend\n```","title":"Examples - Phoenix Integration","ref":"phoenix.html#examples"},{"type":"extras","doc":"# Exceed\n\n`Exceed` is a high-level stream-oriented library for generating Excel files,\nuseful when generating spreadsheets from data sets large enough that they may\nexceed available memory—or the available memory that one wants to dedicate to\nbuilding spreadsheets.","title":"Exceed","ref":"readme.html"},{"type":"extras","doc":"``` elixir\ndef deps do\n  [\n    {:exceed, \"~> 0.6\"}\n  ]\nend\n```","title":"Installation - Exceed","ref":"readme.html#installation"},{"type":"extras","doc":"XLSX files are zip files containing numerous XML files following\n[`SpreadsheetML`](https://learn.microsoft.com/en-us/office/open-xml/spreadsheet/structure-of-a-spreadsheetml-document?tabs=cs)\nOpenXML schemas. [`High-level libraries`](https://hex.pm/packages/elixlsx) already\nexist to generate XLSX files from Elixir, but do not handle streams—all\ncontent to be written to a spreadsheet must be held in memory until the\nspreadsheet is fully written.\n\n[`Low-level libraries`](https://hex.pm/packages/xlsx_stream) already exist to\ngenerate XLSX files from streams, but are so low level that one must know all\nthe ins and outs of SpreasheetML, including positional ordering of files, tags,\nand attributes.\n\nWe wanted a library that hides the complexity of streaming to XML to zlib,\nand hides the complexity of OOXML.","title":"Why a duck? - Exceed","ref":"readme.html#why-a-duck"},{"type":"extras","doc":"XLSX streams are generated by initializing a workbook (with a creator name),\nadding worksheets, and then converting that to a stream.\n\n``` elixir\nstream =\n  Exceed.Workbook.new(\"Creator Name\")\n  |> Exceed.Workbook.add_worksheet(\n    Exceed.Worksheet.new(\"Sheet Name\", [\"Heading 1\", \"Heading 2\"],\n      [[\"Row 1 Cell 1\", \"Row 1 Cell 2\"], [\"Row 2 Cell 1\", \"Row 2 Cell 2\"]])\n  )\n  |> Exceed.stream!()\n\nstream\n|> Stream.into(File.stream!(\"/tmp/workbook.xslx\"))\n|> Stream.run()\n```\n\nWorksheets may be initialized with lists of lists, or they may be initialized\nwith a stream of data that maps to a list of cells.\n\n``` elixir\nrows =\n  Stream.unfold(1, fn\n    10_001 -> nil\n    row_count -> {[\"Row #{row_count} Cell 1\", \"Row #{row_count} Cell 2\"], row_count + 1}\n  end)\n\nExceed.Worksheet.new(\"Sheet Name\", [\"Heading 1\", \"Heading 2\"], rows)\n```","title":"Usage - Exceed","ref":"readme.html#usage"},{"type":"extras","doc":"This library is inspired by and learns from other great libraries. One might\nchoose to use one of those in place of `Exceed`:\n\n- [`elixlsx`](https://hex.pm/packages/elixlsx) - Provides fine-grained control\n  over cells, but is not stream-oriented and thus requires that all source data\n  and rows be retained in memory until the entire workbook is written.\n- [`xlsx_stream`](https://hex.pm/packages/xlsx_stream) - Provides low-level\n  constructs that may be combined to make an Excel file. Works nicely with\n  streams. Requires that one know all the ins and outs of\n  [`SpreadsheetML`](https://learn.microsoft.com/en-us/office/open-xml/spreadsheet/structure-of-a-spreadsheetml-document?tabs=cs)\n  in order to make a valid file that Excel can parse.","title":"Alternatives & References - Exceed","ref":"readme.html#alternatives-references"},{"type":"extras","doc":"This library uses [`medic`](https://github.com/synchronal/medic-rs) for its\ndevelopment workflow.\n\n``` shell\nbrew bundle\n\nbin/dev/doctor\nbin/dev/test\nbin/dev/audit\nbin/dev/update\nbin/dev/shipit\n```\n\nIf one does not want to install extra tooling but wishes to contribute code\nfixes, new features, or documentation, please verify that code is formatted\nwith the versions of Elixir and Erlang specified in `.tool-versions`, passes\nall tests, and passes strict credo and dialyzer.","title":"Contributing - Exceed","ref":"readme.html#contributing"},{"type":"extras","doc":"Copyright 2024 synchronal.dev\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated\ndocumentation files (the \"Software\"), to deal in the Software without restriction, including without limitation the\nrights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit\npersons to whom the Software is furnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the\nSoftware.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE\nWARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR\nCOPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR\nOTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","title":"LICENSE","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}